#include <stdio.h>
#include <stdint.h>
#include "iodefine.h"
#include "cprintf.h"
#include "ivects.h"

/* Type used for encoder-based positions */
typedef long long Position;
/* Type used for speed-based motor control */
typedef long RPM;

extern void bldc_init ();

extern void bldc_exit ();
#define exit(x) bldc_exit()


/* Motor constants */
/* counts per full rotation */
#define MAXC 4000

#define ULEAD (2000 *  3/12)
#define VLEAD (2000 *  7/12)
#define WLEAD (2000 * 11/12)

/* PORT1 / PORTA RGB LED */
#define RED	PORT1.DR.BIT.B1
#define GREEN	PORT1.DR.BIT.B0
#define BLUE	PORTA.DR.BIT.B4

#define DRRED	PORT1.DDR.BIT.B1
#define DRGREEN	PORT1.DDR.BIT.B0
#define DRBLUE	PORTA.DDR.BIT.B4

typedef enum {
  led_black,
  led_red,
  led_green,
  led_yellow,
  led_blue,
  led_magenta,
  led_cyan,
  led_white
} LED_Color;

extern void led_color (LED_Color);
extern void led_flash (LED_Color);
#define led_off() led_color(led_black)

/* PORT4 */
#define THERM	S12AD0.ADDR0A
#define VMOTOR	S12AD0.ADDR1
#define UCURR	S12AD1.ADDR2
#define VCURR	S12AD1.ADDR1
#define WCURR	S12AD1.ADDR0A

/* 110F */
/*#define TOO_HOT ((THERM) > 14468)*/
/* 80c = 175F */
#define TOO_HOT ((THERM) > 37766)

#define UCH	2
#define VCH	1
#define WCH	0

/* PORT7 */
#define UH 0x02
#define UL 0x10
#define VH 0x04
#define VL 0x20
#define WH 0x08
#define WL 0x40

/* PORTB */
#define HALLS	PORTB.PORT.BYTE
#define U_HALL 0x08
#define V_HALL 0x04
#define W_HALL 0x02
#define U2RTS	PORTB.DR.BIT.B7

/* PORT3 */
#define ENC_A	PORT3.PORT.BIT.B3
#define ENC_B	PORT3.PORT.BIT.B2
#define STEP_S	PORT3.PORT.BIT.B1
#define STEP_D	PORT3.PORT.BIT.B0

/* PORT9 */
#define LIMIT1	PORT9.PORT.BIT.B1
#define LIMIT2	PORT9.PORT.BIT.B2
#define LIMIT3	PORT9.PORT.BIT.B3
#define LIMIT4	PORT9.PORT.BIT.B4
#define LIMITS	(PORT9.PORT.BYTE & 0x1e)

extern void limit_pwm_update ();

/* PORTA - also IRQ7-A */
#define ENC_Z		PORTA.PORT.BIT.B5

#define SD_LIMIT1	PORTA.DR.BIT.B3
#define SD_LIMIT2	PORTA.DR.BIT.B2
#define SD_FAULT	PORTB.DR.BIT.B0
#define SD_ESTOP_IN	PORTE.PORT.BIT.B2
#define SD_ESTOP_OUT	PORT2.DR.BIT.B4
#define SD_ESTOP_DDR	PORT2.DDR.BIT.B4

/* PORTB */
#define U2_CTS		PORTB.PORT.BIT.B4
#define U2_RTS		PORTB.DR.BIT.B7

/* PORTD */
#define U1_DTR		PORTD.PORT.BIT.B7
#define U1_CTS		PORTD.PORT.BIT.B6
#define U1_RTS		PORTD.DR.BIT.B4

extern volatile long msec;
extern void wait_msec (int m);

#include "motor.h"
#include "config.h"

void encoder_init ();
/* 0..3999 */
int encoder_val ();
int encoder_synced ();
int encoder_reset ();
/* 4000 counts per rotation, no fixed "zero" point.  */
Position encoder_lval ();
int encoder_ang (); /* 0..1999 relative to the index pulse */

extern volatile int8_t encoder_direction;
extern int encoder_rpm ();

extern char pwm_status[];
extern int pwm_status_new;

/* position "encoder position scale" */
double dcos (int position);

/* 75% duty gives us time to recharge the bootstrap caps */
#define MAX_TORQUE	24578
void cpwm_init ();
/* Levels -32768..32767 */
void cpwm_levels (int u, int v, int w);
/* Level -32768..32767 */
void cpwm_torque (int t);

/* used for quenching winding currents during motor stop */
#define STOP_MARG 400
#define STOP_LEV(CUR,zer) \
  (CUR > zer+STOP_MARG ? MAX_TORQUE :		\
   CUR < zer-STOP_MARG ? -MAX_TORQUE :		\
   0)

void bldc_stop ();
void bldc_run ();

void simple_commutate (int forever);

#define STEP_COUNT MTU2.TCNT
void step_direction_init ();
/* Counts "user" input.  */
long long step_direction_lval ();
void step_direction_reset ();

void thermistor_init (); /* also for motor volts */
float temperature (); /* F */
float motor_volts (); /* V */
extern int uzero, vzero, wzero;
extern volatile int raw_current;
extern float u_curr, v_curr, w_curr;
extern float cos_angle, sin_angle;
extern volatile float motor_current;
extern volatile int cpwm_torque_applied;

#define rx_psw() __builtin_rx_mvfc(0)
#define rx_enable()	__builtin_rx_setpsw('I')
#define rx_disable()	__builtin_rx_clrpsw('I')

typedef struct {
  short off_by;
  short rpm;
  short d_rpm;
  short milliamps;
  short d_milliamps;
  short torque;
  char state;
} State;
#define NSTATE 200
extern State state[NSTATE+2];
extern int state_n;
