#include "bldc.h"
#include "r_flash_api_rx600.h"

/*-----------------------------------------------------------------------------*/

#define IS_BLANK(block) R_FlashDataAreaBlankCheck (DF_PAGE(block), BLANK_CHECK_8_BYTE) == FLASH_BLANK

static int initted = 0;

int
config_valid ()
{
  if (!initted)
    {
      initted = 1;
      R_FlashDataAreaAccess (~0, ~0);
    }
  return (1
	  && config_wiring_done ()
	  && config_axis_done ()
	  && config_limits_done ()
	  /* && config_gearing_done () */
	  && config_steps_done ()
	  && config_home_rpm_done ()
	  /* && config_uart2_done () */
	  && config_motor_calib_done ()
	  );
}

int
config_wiring_done ()
{
  return ! IS_BLANK (P_AXIS);
}

int
config_axis_done ()
{
  return ! IS_BLANK (P_AXIS);
}

int
config_limits_done ()
{
  return ! IS_BLANK (P_LIMITS);
}

int
config_gearing_done ()
{
  return ! IS_BLANK (P_GEARING);
}

int
config_steps_done ()
{
  return ! IS_BLANK (P_STEPS);
}

int
config_home_rpm_done ()
{
  return ! IS_BLANK (P_HOMERPM);
}

int
config_uart2_done ()
{
  return ! IS_BLANK (P_UART2);
}

int
config_motor_calib_done ()
{
  return ! IS_BLANK (P_MOTORCAL);
}

/*-----------------------------------------------------------------------------*/

Config_Wiring *
config_wiring_get ()
{
  return DF_PTR (Config_Wiring, P_WIRING);
}

Config_Axis *
config_axis_get ()
{
  return DF_PTR (Config_Axis, P_AXIS);
}

Config_Limits *
config_limits_get ()
{
  return DF_PTR (Config_Limits, P_LIMITS);
}

Config_Gearing *
config_gearing_get ()
{
  return DF_PTR (Config_Gearing, P_GEARING);
}

Config_Steps *
config_steps_get ()
{
  return DF_PTR (Config_Steps, P_STEPS);
}

Config_Home_RPM *
config_home_rpm_get ()
{
  return DF_PTR (Config_Home_RPM, P_HOMERPM);
}

Config_UART2 *
config_uart2_get ()
{
  return DF_PTR (Config_UART2, P_UART2);
}

Config_Motor_Calib *
config_motor_calib_get ()
{
  return DF_PTR (Config_Motor_Calib, P_MOTORCAL);
}

/*-----------------------------------------------------------------------------*/

static void
SAVE (void *cfg, int nbytes, int block)
{
  int fbytes;
  int32_t cfga = (int32_t) cfg;
  int r;

  nbytes = (nbytes + 7) & ~7;

  while (nbytes > 0)
    {
      int i;
      fbytes = (nbytes > 2048) ? 2048 : nbytes;
      for (i=0; i<10; i++)
	{
	  if (! IS_BLANK (DF_BLOCK(block)))
	    R_FlashErase (DF_BLOCK(block));
	  r = R_FlashWrite (DF_PAGE (block), cfga, fbytes);
	  if (r == FLASH_SUCCESS)
	    break;
	  wait_msec (10);
	}
      if (r != FLASH_SUCCESS)
	{
	  printf("flash ret %d...", r);
	  tty_getc();
	}
      block ++;
      cfg += fbytes;
      nbytes -= fbytes;
    }
}

void
config_wiring_save (Config_Wiring *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_WIRING);
}

void
config_axis_save (Config_Axis *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_AXIS);
}

void
config_limits_save (Config_Limits *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_LIMITS);
}

void
config_gearing_save (Config_Gearing *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_GEARING);
}

void
config_steps_save (Config_Steps *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_STEPS);
}

void
config_home_rpm_save (Config_Home_RPM *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_HOMERPM);
}

void
config_uart2_save (Config_UART2 *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_UART2);
}

void
config_motor_calib_save (Config_Motor_Calib *cfg)
{
  SAVE (cfg, sizeof(*cfg), P_MOTORCAL);
}

