#include <stdio.h>
#include <math.h>
#include "bldc.h"

extern int cpwm_step;

double
dcos (double pos)
{
  return cos (pos * M_PI / 90);
}

#define MINWAIT 300

main()
{
  int i;
  int halls, old_halls = -1;
  int u = 0;
  int v = 0;
  int w = 0;
  int power = 0;
  int pos, old_pos = -999;
  int running = 0;
  int update_status = 1;
  int then = 0;
  int direction = 1;
  int hall;

  bldc_init ();

  tty_puts("\033[1;1H\033[2J\n\nHello from Sine 1!\n");

  /* Shut off the motor.  */
  PORTB.DR.BYTE = UL | VL | WL;

  printf("Looping...\n");
  rx_enable();

  while (1)
    {

      if (tty_ready ())
	{
	  int c = tty_getc ();
	  iprintf("char %d\n", c);
	  switch (c)
	    {
	    case 'r':
	      bldc_run ();
	      old_pos = -999;
	      running = 1;
	      break;

	    case 's':
	    case ' ':
	      bldc_stop ();
	      old_pos = -999;
	      running = 0;
	      RED = BLUE = GREEN = 0;
	      break;

	    case '0' ... '9':
	      if (c == '0')
		power = 32767;
	      else
		power = (c - '0') * 32767/10;
	      old_pos = -999;
	      break;

	    case '-':
	    case '_':
	      direction = -1;
	      break;
	    case '+':
	    case '=':
	      direction = 1;
	      break;
	    }
	}

      pos = encoder_ang ();

      if (pos != old_pos)
	{
	  old_pos = pos;
	  update_status = 1;
	  cpwm_torque (power * direction);

	  if (running)
	    {
	      hall = PORTB.PORT.BYTE;
	      if (hall & U_HALL)
		RED = 1;
	      else
		RED = 0;
	      if (hall & V_HALL)
		BLUE = 1;
	      else
		BLUE = 0;
	      if (hall & W_HALL)
		GREEN = 1;
	      else
		GREEN = 0;
	    }

	}

      if (tty_idle () && (msec - then) > MINWAIT)
	{
	  int rpm = encoder_rpm ();
	  int ang = encoder_ang ();
	  int lpos = encoder_lval ();
	  then = msec;
	  update_status = 0;
	  printf("\033[1;1H  pos = %5d, ang = %5d, lpos = %d, \033[%dmpower = %5d\033[0m,  RPM = %5d  step %3d ",
		 pos, ang, lpos, running ? 32 : 31, power, rpm, cpwm_step);

	  temperature();
	  printf(" VMotor = %5.1f, %5.3f ", motor_volts (), motor_current);

	  printf("\033[K");
	  fflush (stdout);
	}
    }

  exit (0);
}
